#include-once

; #INDEX# =======================================================================================================================
; Title .........: I2D_Nano
; AutoIt Version : 3.3.14.1
; Date ..........: 22.10.2015
; Modified ......: 10.12.2015 - Added  _I2D_BufferCreateFromFile
;                             - Added  _I2D_BufferSaveToFile
;                             - BugFix _I2D_BufferCreateFromBitmap
;                             - Added  Au3.api fr die Autovervollstndigung
; Language ......: Deutsch
; Description ...: Methoden zur Erzeugung und Lschung von I2D Structs und weitere Kleinigkeiten
; Author(s) .....: Mars
; ===============================================================================================================================

#include 'DllStructProtected.au3'
#include <GDIPlus.au3>

; #DEF# =========================================================================================================================
; I2D bedeutet Image 2D
; int[0] - GFX -> GDIPlusGraphics des Buffers
; int[1] - BMP -> GDIPlusBitmap des Buffers
; int[2] - PTR -> Pointer auf die Bitmapdaten (mit ASM/Structs ansprechbar)
; int[3] - WIDTH  -> Breite des Buffers
; int[4] - HEIGHT -> Hhe des Buffers
; Sinn der Sache ist es eine einfache Verbindung zwischen GDI+ und Structs/ASM herzustellen.
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
; _I2D_BufferCreate
; _I2D_BufferCreateFromBitmap
; _I2D_BufferCreateFromFile
; _I2D_BufferDelete
; _I2D_BufferSaveToFile
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Erzeugt eine I2D Struct
; ===============================================================================================================================
Func _I2D_BufferCreate($iW, $iH)
	Local $vIMG = DllStructCreateProtected('int GFX;int BMP;int PTR;uint WIDTH;uint HEIGHT'), _
	$xTMP = DllStructSetData($vIMG, 'WIDTH', $iW) * DllStructSetData($vIMG, 'HEIGHT', $iH), _
	$vBMP = DllStructCreateProtected('int[' & $xTMP & ']'), _
	$xTMP = DllStructSetData($vIMG, 'PTR', DllStructGetPtr($vBMP)) + DllStructSetData($vIMG, 'BMP', DllCall($__g_hGDIPDll, 'uint', 'GdipCreateBitmapFromScan0', 'int', $vIMG.WIDTH, 'int', $vIMG.HEIGHT, 'int', $vIMG.WIDTH * 4, 'int', 0x0026200A, 'ptr', $vIMG.PTR, 'int*', 0)[6]) + DllStructSetData($vIMG, 'GFX', DllCall($__g_hGDIPDll, 'int', 'GdipGetImageGraphicsContext', 'handle', $vIMG.BMP, 'ptr*', 0)[2])
	Return $vIMG
EndFunc

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Erzeugt eine I2D Struct von einer GDI+ Bitmap
; ===============================================================================================================================
Func _I2D_BufferCreateFromBitmap($hBMP)
	Local $iW = _GDIPlus_ImageGetWidth($hBMP), $iH = _GDIPlus_ImageGetHeight($hBMP), $vIMG = _I2D_BufferCreate($iW, $iH)
	DllCall($__g_hGDIPDll, 'int', 'GdipDrawImageRectI', 'handle', $vIMG.GFX, 'handle', $hBMP, 'int', 0, 'int', 0, 'int', $vIMG.WIDTH, 'int', $vIMG.HEIGHT)
	Return $vIMG
EndFunc

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Erzeugt eine I2D Struct von einer Datei (Bild)
; ===============================================================================================================================
Func _I2D_BufferCreateFromFile($sFile)
	If Not FileExists($sFile) Then Return SetError(1, 0, False)
	Local $hBMP = _GDIPlus_ImageLoadFromFile($sFile)
	If @error Then Return SetError(2, 0, False)
	Local $vIMG = _I2D_BufferCreateFromBitmap($hBMP)
	_GDIPlus_ImageDispose($hBMP)
	Return $vIMG
EndFunc

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Lscht eine I2D Struct
; ===============================================================================================================================
Func _I2D_BufferDelete(ByRef $vIMG)
	If Not IsDllStruct($vIMG) Then Return SetError(1)
	DllCall($__g_hGDIPDll, 'int', 'GdipDeleteGraphics', 'handle', $vIMG.GFX)
	DllCall($__g_hGDIPDll, 'int', 'GdipDisposeImage', 'handle', $vIMG.BMP)
	DllStructDelete($vIMG.PTR)
	DllStructDelete($vIMG)
	Return True
EndFunc

; #FUNCTION# ====================================================================================================================
; Author ........: Mars
; Description ...: Speichert eine I2D Struct in einer Datei (Bild)
; ===============================================================================================================================
Func _I2D_BufferSaveToFile($vIMG, $sFile)
	If Not IsDllStruct($vIMG) Then Return SetError(1, 0, False)
	_GDIPlus_ImageSaveToFile($vIMG.BMP, $sFile)
EndFunc

#CS Au3.api
_I2D_BufferCreate ( $iW, $iH ) Erzeugt eine I2D Struct
_I2D_BufferCreateFromBitmap ( $hBMP ) Erzeugt eine I2D Struct von einer GDI+ Bitmap
_I2D_BufferCreateFromFile ( $sFile ) Erzeugt eine I2D Struct (via GDI+) von einer Datei (Bild)
_I2D_BufferDelete ( ByRef $vIMG ) Lscht eine I2D Struct
_I2D_BufferSaveToFile ( $vIMG, $sFile ) Speichert eine I2D Struct (via GDI+) in einer Datei (Bild)
#CE Au3.api